@@ -25,10 +25,10 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 2427 2007-05-27 19:23:28Z wwalker $"
-__version__   = "$Revision: 2427 $"
-__date__      = "$Date: 2007-05-27 15:23:28 -0400 (Sun, 27 May 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: braille.py 3267 2007-11-29 15:32:25Z wwalker $"
+__version__   = "$Revision: 3267 $"
+__date__      = "$Date: 2007-11-29 10:32:25 -0500 (Thu, 29 Nov 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -36,7 +36,7 @@ log = logging.getLogger("braille")
 
 import signal
 
-import atspi
+import pyatspi
 
 # We'll use the official BrlAPI pythons (as of BrlTTY 3.8) if they
 # are available.  Otherwise, we'll fall back to our own bindings.
@@ -69,8 +69,6 @@ import orca_state
 import settings
 
 from orca_i18n import _                          # for gettext support
-from rolenames import getShortBrailleForRoleName # localized role names
-from rolenames import getLongBrailleForRoleName  # localized role names
 
 # If True, this module has been initialized.
 #
@@ -153,8 +151,8 @@ CMD_RESTARTBRL        = 0x43
 CMD_RESTARTSPEECH     = 0x44
 CMD_MAX               = 0x44
 
-BRL_FLG_REPEAT_INITIAL= 0x800000
-BRL_FLG_REPEAT_DELAY  = 0x400000
+BRL_FLG_REPEAT_INITIAL = 0x800000
+BRL_FLG_REPEAT_DELAY   = 0x400000
 
 # Common names for most used BrlTTY commands, to be shown in the GUI:
 # ATM, the ones used in default.py are:
@@ -292,6 +290,16 @@ class Region:
         been scrolled off the display."""
         pass
 
+    def getAttributeMask(self):
+        """Creates a string which can be used as the attrOr field of brltty's
+        write structure for the purpose of indicating text attributes and
+        selection."""
+
+        # Create an empty mask.
+        #
+        mask = ['\x00'] * len(self.string)
+        return "".join(mask)
+
 class Component(Region):
     """A subclass of Region backed by an accessible.  This Region will react
     to any cursor routing key events and perform the default action on the
@@ -317,15 +325,12 @@ class Component(Region):
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
-        actions = self.accessible.action
-        if actions:
-            actions.doAction(0)
-        else:
-
-            # [[[WDW - HACK to do a mouse button 1 click if we have
-            # to.  For example, page tabs don't have any actions but
-            # we want to be able to select them with the cursor
-            # routing key.]]]
+        try:
+            action = self.accessible.queryAction()
+        except:
+            # Do a mouse button 1 click if we have to.  For example, page tabs
+            # don't have any actions but we want to be able to select them with
+            # the cursor routing key.
             #
             debug.println(debug.LEVEL_FINEST,
                           "braille.Component.processRoutingKey: no action")
@@ -333,6 +338,8 @@ class Component(Region):
                 eventsynthesizer.clickObject(self.accessible, 1)
             except:
                 debug.printException(debug.LEVEL_SEVERE)
+        else:
+            action.doAction(0)
 
 class Text(Region):
     """A subclass of Region backed by a Text object.  This Region will
@@ -365,15 +372,16 @@ class Text(Region):
             self.caretOffset = 0
             self.lineOffset = 0
             [string, startOffset, endOffset] = \
-                self.accessible.text.getTextAtOffset(
+                self.accessible.queryText().getTextAtOffset(
                     0,
-                    atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+                    pyatspi.TEXT_BOUNDARY_LINE_START)
 
         cursorOffset = min(self.caretOffset - self.lineOffset, len(string))
 
         self._maxCaretOffset = self.lineOffset + len(string.decode("UTF-8"))
 
-        string = string + eol
+        self.eol = eol
+        string = string + self.eol
 
         self.label = label
         if self.label:
@@ -416,7 +424,109 @@ class Text(Region):
                 return
 
         newCaretOffset = min(self.lineOffset + offset, self._maxCaretOffset)
-        self.accessible.text.setCaretOffset(newCaretOffset)
+        self.accessible.queryText().setCaretOffset(newCaretOffset)
+
+    def getAttributeMask(self):
+        """Creates a string which can be used as the attrOr field of brltty's
+        write structure for the purpose of indicating text attributes and
+        selection."""
+
+        try:
+            text = self.accessible.queryText()
+        except NotImplementedError:
+            return ''
+
+        # Start with an empty mask.
+        #
+        stringLength = len(self.string) - len(self.eol)
+        attrMask = ['\x00']*stringLength
+
+        attrIndicator = settings.textAttributesBrailleIndicator
+        selIndicator = settings.brailleSelectorIndicator
+        indicateAttr = (attrIndicator != settings.TEXT_ATTR_BRAILLE_NONE)
+        indicateSel = (selIndicator != settings.BRAILLE_SEL_NONE)
+
+        if indicateAttr and settings.enabledBrailledTextAttributes:
+            # Identify what attributes the user cares about.  Also get the
+            # default attributes for the text object because those attributes
+            # may not be included as attributes for the character.
+            #
+            script = orca_state.activeScript
+            enabledAttributes = settings.enabledBrailledTextAttributes
+            [userAttrList, userAttrDict] = \
+                script.textAttrsToDictionary(enabledAttributes)
+            defaultAttributes = text.getDefaultAttributes()
+            [defaultAttrList, defaultAttrDict] = \
+                script.textAttrsToDictionary(defaultAttributes)
+
+            # [[[TODO - HACK - JD: The end offset returned by SO/OOo is the
+            # offset of the first character that doesn't have the attribute
+            # in question; all other apps return the offset of the last
+            # character that has that attribute]]]
+            #
+            adjustment = 0
+            appName = orca_state.locusOfFocus.getApplication().name
+            if appName.startswith("soffice"):
+                adjustment += 1
+
+            i = self.lineOffset
+            endOffset = self.lineOffset + stringLength
+            while i < endOffset - 1:
+                for attribute in userAttrList:
+                    defaultValue = None
+                    if attribute in defaultAttrList:
+                        defaultValue = defaultAttrDict[attribute]
+                    [value, start, end, defined] = \
+                            text.getAttributeValue(i, attribute)
+
+                    notOfInterest = userAttrDict[attribute]
+                    if defined:
+                        weCare = (value != notOfInterest)
+                    elif defaultValue and len(notOfInterest):
+                        weCare = (defaultValue != notOfInterest)
+                    else:
+                        weCare = False
+
+                    if weCare and start >= 0:
+                        maskStart = start - self.lineOffset
+                        maskEnd = end - self.lineOffset - adjustment
+                        maskLength = maskEnd - maskStart
+                        attrMask[maskStart:maskEnd] = \
+                                                  [attrIndicator] * maskLength
+                if (end <= i):
+                    break
+
+                i = end
+
+        if indicateSel:
+            nSelections = text.getNSelections()
+            if nSelections:
+                for i in range(0, nSelections):
+                    [start, end] = text.getSelection(i)
+                    maskStart = max(self.lineOffset, start) - self.lineOffset
+                    maskEnd = min(end - self.lineOffset, stringLength)
+                    # Combine the selection indicator with the attribute
+                    # indicator.
+                    #
+                    for j in range(maskStart, maskEnd):
+                        if attrMask[j] != '\x00' \
+                           and attrMask[j] != selIndicator:
+                            attrMask[j] = '\xc0'
+                        else:
+                            attrMask[j] = selIndicator
+
+        mask = "".join(attrMask)
+
+        # Add empty mask characters for the EOL character as well as for
+        # any label that might be present.
+        #
+        for i in range (0, len(self.eol)):
+            mask += '\x00'
+        if self.label:
+            for i in range (0, len(self.label) + 1):
+                mask = '\x00' + mask
+
+        return mask
 
 class ReviewComponent(Component):
     """A subclass of Component that is to be used for flat review mode."""
@@ -464,7 +574,7 @@ class ReviewText(Region):
         been scrolled off the display."""
 
         newCaretOffset = self.lineOffset + offset
-        self.accessible.text.setCaretOffset(newCaretOffset)
+        self.accessible.queryText().setCaretOffset(newCaretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -489,11 +599,12 @@ class Line:
         If the region with focus is not on this line, then the index
         will be -1.
 
-        Returns [string, offsetIndex]
+        Returns [string, offsetIndex, attributeMask]
         """
 
         string = ""
         focusOffset = -1
+        attributeMask = ""
         for region in self.regions:
             if region == _regionWithFocus:
                 focusOffset = len(string.decode("UTF-8"))
@@ -503,8 +614,10 @@ class Line:
                 # BrlTTY.]]]
                 #
                 string += region.string.replace("\342\200\246", "...")
+            mask = region.getAttributeMask()
+            attributeMask += mask
 
-        return [string, focusOffset]
+        return [string, focusOffset, attributeMask]
 
     def getRegionAtOffset(self, offset):
         """Finds the Region at the given 0-based offset in this line.
@@ -518,10 +631,11 @@ class Line:
         # for a region, and then pass the event off to the region for
         # handling.
         #
-        region = None
+        foundRegion = None
         string = ""
         pos = 0
         for region in self.regions:
+            foundRegion = region
             string = string + region.string
             if len(string.decode("UTF-8")) > offset:
                 break
@@ -531,7 +645,7 @@ class Line:
         if offset >= len(string.decode("UTF-8")):
             return [None, -1]
         else:
-            return [region, offset - pos]
+            return [foundRegion, offset - pos]
 
     def processRoutingKey(self, offset):
         """Processes a cursor routing key press on this Component.  The offset
@@ -584,8 +698,6 @@ def addLine(line):
     - line: an instance of Line to add.
     """
 
-    global _lines
-
     _lines.append(line)
     line._index = len(_lines)
 
@@ -631,7 +743,7 @@ def setFocus(region, panToFocus=True):
             lineNum += 1
 
     line = _lines[_viewport[1]]
-    [string, offset] = line.getLineInfo()
+    [string, offset, attributeMask] = line.getLineInfo()
 
     # If the cursor is too far right, we scroll the viewport
     # so the cursor will be on the last cell of the display.
@@ -686,7 +798,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     # actually is in the string.
     #
     line = _lines[_viewport[1]]
-    [string, focusOffset] = line.getLineInfo()
+    [string, focusOffset, attributeMask] = line.getLineInfo()
     cursorOffset = -1
     if focusOffset >= 0:
         cursorOffset = focusOffset + _regionWithFocus.cursorOffset
@@ -718,30 +830,25 @@ def refresh(panToCursor=True, targetCursorCell=0):
     else:
         cursorCell += 1 # Normalize to 1-based offset
 
-    debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
-    log.info("line:'%s'" % string)
-
-    debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
-                  % (string[startPos:endPos], cursorCell))
-    log.info("visible:'%s'" % string[startPos:endPos])
-    log.info("cursor:%d" % cursorCell)
+    logLine = "BRAILLE LINE:  '%s'" % string
+    debug.println(debug.LEVEL_INFO, logLine)
+    log.info(logLine)
+    logLine = "     VISIBLE:  '%s', cursor=%d" % \
+                    (string[startPos:endPos], cursorCell)
+    debug.println(debug.LEVEL_INFO, logLine)
+    log.info(logLine)
 
     string = string.decode("UTF-8")
     substring = string[startPos:endPos].encode("UTF-8")
     if useBrlAPIBindings:
         if brlAPIRunning:
-            try:
-                # The name after (and including) BrlTTY v3.8 revision 2810
-                #
-                writeStruct = brlapi.WriteStruct()
-            except:
-                # The name before BrlTTY v3.8 revision 2810
-                #
-                writeStruct = brlapi.Write()
+            writeStruct = brlapi.WriteStruct()
             writeStruct.regionBegin = 1
             writeStruct.regionSize = len(substring.decode("UTF-8"))
             while writeStruct.regionSize < _displaySize[0]:
                 substring += " "
+                if attributeMask:
+                    attributeMask += '\x00'
                 writeStruct.regionSize += 1
             writeStruct.text = substring
             writeStruct.cursor = cursorCell
@@ -762,6 +869,9 @@ def refresh(panToCursor=True, targetCursorCell=0):
             #    myUnderline += '\xc0'
             #writeStruct.attrOr = myUnderline
 
+            if attributeMask:
+                writeStruct.attrOr = attributeMask[startPos:endPos]
+
             brlAPI.write(writeStruct)
     else:
         brl.writeText(cursorCell, substring)
@@ -770,7 +880,11 @@ def refresh(panToCursor=True, targetCursorCell=0):
         if not monitor:
             monitor = brlmon.BrlMon(_displaySize[0])
             monitor.show_all()
-        monitor.writeText(cursorCell, substring)
+        if attributeMask:
+            subMask = attributeMask[startPos:endPos]
+        else:
+            subMask = None
+        monitor.writeText(cursorCell, substring, subMask)
     elif monitor:
         monitor.destroy()
 
@@ -854,7 +968,7 @@ def panRight(panAmount=0):
     if len(_lines) > 0:
         lineNum = _viewport[1]
         newX = _viewport[0] + panAmount
-        [string, focusOffset] = _lines[lineNum].getLineInfo()
+        [string, focusOffset, attributeMask] = _lines[lineNum].getLineInfo()
         if newX < len(string.decode("UTF-8")):
             _viewport[0] = newX
 
@@ -943,27 +1057,27 @@ def _processBrailleEvent(command):
     return consumed
 
 def _brlAPIKeyReader(source, condition):
-        """Method to read a key from the BrlAPI bindings.  This is a
-        gobject IO watch handler.
-        """
-        key = brlAPI.readKey(False)
-        if key:
-            flags = key >> 32
-            lower = key & 0xFFFFFFFF
-            keyType = lower >> 29
-            keyCode = lower & 0x1FFFFFFF
+    """Method to read a key from the BrlAPI bindings.  This is a
+    gobject IO watch handler.
+    """
+    key = brlAPI.readKey(False)
+    if key:
+        #flags = key >> 32
+        lower = key & 0xFFFFFFFF
+        #keyType = lower >> 29
+        keyCode = lower & 0x1FFFFFFF
 
-            # [[TODO: WDW - HACK If we have a cursor routing key, map
-            # it back to the code we used to get with earlier versions
-            # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
-            # routing key instead of 0x1000).  This may change before
-            # the offical BrlAPI Python bindings are released.]]]
-            #
-            if keyCode & 0x10000:
-                keyCode = 0x100 | (keyCode & 0xFF)
-            if keyCode:
-                _processBrailleEvent(keyCode)
-        return brlAPIRunning
+        # [[TODO: WDW - HACK If we have a cursor routing key, map
+        # it back to the code we used to get with earlier versions
+        # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
+        # routing key instead of 0x1000).  This may change before
+        # the offical BrlAPI Python bindings are released.]]]
+        #
+        if keyCode & 0x10000:
+            keyCode = 0x100 | (keyCode & 0xFF)
+        if keyCode:
+            _processBrailleEvent(keyCode)
+    return brlAPIRunning
 
 def setupKeyRanges(keys):
     """Hacky method to tell BrlTTY what to send and not send us via
