@@ -23,15 +23,13 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3199 2007-10-29 16:33:57Z richb $"
-__version__   = "$Revision: 3199 $"
-__date__      = "$Date: 2007-10-29 12:33:57 -0400 (Mon, 29 Oct 2007) $"
+__id__        = "$Id: braillegenerator.py 3278 2007-11-29 19:07:04Z shaeger $"
+__version__   = "$Revision: 3278 $"
+__date__      = "$Date: 2007-11-29 14:07:04 -0500 (Thu, 29 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import math
-
-import atspi
+import pyatspi
 import braille
 import debug
 import orca_state
@@ -46,10 +44,10 @@ class BrailleGenerator:
     primary entry point.  Subclasses can feel free to override/extend
     the brailleGenerators instance field as they see fit."""
 
-    SKIP_CONTEXT_ROLES = (rolenames.ROLE_MENU,
-                          rolenames.ROLE_MENU_BAR,
-                          rolenames.ROLE_PAGE_TAB_LIST,
-                          rolenames.ROLE_COMBO_BOX)
+    SKIP_CONTEXT_ROLES = (pyatspi.ROLE_MENU,
+                          pyatspi.ROLE_MENU_BAR,
+                          pyatspi.ROLE_PAGE_TAB_LIST,
+                          pyatspi.ROLE_COMBO_BOX)
 
     def __init__(self, script):
 
@@ -59,103 +57,99 @@ class BrailleGenerator:
         self._script = script
 
         self.brailleGenerators = {}
-        self.brailleGenerators[rolenames.ROLE_ALERT]               = \
+        self.brailleGenerators[pyatspi.ROLE_ALERT]               = \
              self._getBrailleRegionsForAlert
-        self.brailleGenerators[rolenames.ROLE_ANIMATION]           = \
+        self.brailleGenerators[pyatspi.ROLE_ANIMATION]           = \
              self._getBrailleRegionsForAnimation
-        self.brailleGenerators[rolenames.ROLE_ARROW]               = \
+        self.brailleGenerators[pyatspi.ROLE_ARROW]               = \
              self._getBrailleRegionsForArrow
-        self.brailleGenerators[rolenames.ROLE_CHECK_BOX]           = \
+        self.brailleGenerators[pyatspi.ROLE_CHECK_BOX]           = \
              self._getBrailleRegionsForCheckBox
-        self.brailleGenerators[rolenames.ROLE_CHECK_MENU]          = \
+        self.brailleGenerators[pyatspi.ROLE_CHECK_MENU_ITEM]     = \
              self._getBrailleRegionsForCheckMenuItem
-        self.brailleGenerators[rolenames.ROLE_CHECK_MENU_ITEM]     = \
-             self._getBrailleRegionsForCheckMenuItem
-        self.brailleGenerators[rolenames.ROLE_COLUMN_HEADER]       = \
+        self.brailleGenerators[pyatspi.ROLE_COLUMN_HEADER]       = \
              self._getBrailleRegionsForColumnHeader
-        self.brailleGenerators[rolenames.ROLE_COMBO_BOX]           = \
+        self.brailleGenerators[pyatspi.ROLE_COMBO_BOX]           = \
              self._getBrailleRegionsForComboBox
-        self.brailleGenerators[rolenames.ROLE_DESKTOP_ICON]        = \
+        self.brailleGenerators[pyatspi.ROLE_DESKTOP_ICON]        = \
              self._getBrailleRegionsForDesktopIcon
-        self.brailleGenerators[rolenames.ROLE_DIAL]                = \
+        self.brailleGenerators[pyatspi.ROLE_DIAL]                = \
              self._getBrailleRegionsForDial
-        self.brailleGenerators[rolenames.ROLE_DIALOG]              = \
+        self.brailleGenerators[pyatspi.ROLE_DIALOG]              = \
              self._getBrailleRegionsForDialog
-        self.brailleGenerators[rolenames.ROLE_DIRECTORY_PANE]      = \
+        self.brailleGenerators[pyatspi.ROLE_DIRECTORY_PANE]      = \
              self._getBrailleRegionsForDirectoryPane
-        self.brailleGenerators[rolenames.ROLE_FRAME]               = \
+        self.brailleGenerators[pyatspi.ROLE_FRAME]               = \
              self._getBrailleRegionsForFrame
-        self.brailleGenerators[rolenames.ROLE_HTML_CONTAINER]      = \
+        self.brailleGenerators[pyatspi.ROLE_HTML_CONTAINER]      = \
              self._getBrailleRegionsForHtmlContainer
-        self.brailleGenerators[rolenames.ROLE_ICON]                = \
+        self.brailleGenerators[pyatspi.ROLE_ICON]                = \
              self._getBrailleRegionsForIcon
-        self.brailleGenerators[rolenames.ROLE_IMAGE]               = \
+        self.brailleGenerators[pyatspi.ROLE_IMAGE]               = \
              self._getBrailleRegionsForImage
-        self.brailleGenerators[rolenames.ROLE_LABEL]               = \
+        self.brailleGenerators[pyatspi.ROLE_LABEL]               = \
              self._getBrailleRegionsForLabel
-        self.brailleGenerators[rolenames.ROLE_LIST]                = \
+        self.brailleGenerators[pyatspi.ROLE_LIST]                = \
              self._getBrailleRegionsForList
-        self.brailleGenerators[rolenames.ROLE_MENU]                = \
+        self.brailleGenerators[pyatspi.ROLE_MENU]                = \
              self._getBrailleRegionsForMenu
-        self.brailleGenerators[rolenames.ROLE_MENU_BAR]            = \
+        self.brailleGenerators[pyatspi.ROLE_MENU_BAR]            = \
              self._getBrailleRegionsForMenuBar
-        self.brailleGenerators[rolenames.ROLE_MENU_ITEM]           = \
+        self.brailleGenerators[pyatspi.ROLE_MENU_ITEM]           = \
              self._getBrailleRegionsForMenuItem
-        self.brailleGenerators[rolenames.ROLE_OPTION_PANE]         = \
+        self.brailleGenerators[pyatspi.ROLE_OPTION_PANE]         = \
              self._getBrailleRegionsForOptionPane
-        self.brailleGenerators[rolenames.ROLE_PAGE_TAB]            = \
+        self.brailleGenerators[pyatspi.ROLE_PAGE_TAB]            = \
              self._getBrailleRegionsForPageTab
-        self.brailleGenerators[rolenames.ROLE_PAGE_TAB_LIST]       = \
+        self.brailleGenerators[pyatspi.ROLE_PAGE_TAB_LIST]       = \
              self._getBrailleRegionsForPageTabList
-        self.brailleGenerators[rolenames.ROLE_PANEL]               = \
+        self.brailleGenerators[pyatspi.ROLE_PANEL]               = \
              self._getBrailleRegionsForPanel
-        self.brailleGenerators[rolenames.ROLE_PARAGRAPH]           = \
+        self.brailleGenerators[pyatspi.ROLE_PARAGRAPH]           = \
              self._getBrailleRegionsForText
-        self.brailleGenerators[rolenames.ROLE_PASSWORD_TEXT]       = \
+        self.brailleGenerators[pyatspi.ROLE_PASSWORD_TEXT]       = \
              self._getBrailleRegionsForText
-        self.brailleGenerators[rolenames.ROLE_PROGRESS_BAR]        = \
+        self.brailleGenerators[pyatspi.ROLE_PROGRESS_BAR]        = \
              self._getBrailleRegionsForProgressBar
-        self.brailleGenerators[rolenames.ROLE_PUSH_BUTTON]         = \
+        self.brailleGenerators[pyatspi.ROLE_PUSH_BUTTON]         = \
              self._getBrailleRegionsForPushButton
-        self.brailleGenerators[rolenames.ROLE_RADIO_BUTTON]        = \
+        self.brailleGenerators[pyatspi.ROLE_RADIO_BUTTON]        = \
              self._getBrailleRegionsForRadioButton
-        self.brailleGenerators[rolenames.ROLE_RADIO_MENU]          = \
+        self.brailleGenerators[pyatspi.ROLE_RADIO_MENU_ITEM]     = \
              self._getBrailleRegionsForRadioMenuItem
-        self.brailleGenerators[rolenames.ROLE_RADIO_MENU_ITEM]     = \
-             self._getBrailleRegionsForRadioMenuItem
-        self.brailleGenerators[rolenames.ROLE_ROW_HEADER]          = \
+        self.brailleGenerators[pyatspi.ROLE_ROW_HEADER]          = \
              self._getBrailleRegionsForRowHeader
-        self.brailleGenerators[rolenames.ROLE_SCROLL_BAR]          = \
+        self.brailleGenerators[pyatspi.ROLE_SCROLL_BAR]          = \
              self._getBrailleRegionsForScrollBar
-        self.brailleGenerators[rolenames.ROLE_SLIDER]              = \
+        self.brailleGenerators[pyatspi.ROLE_SLIDER]              = \
              self._getBrailleRegionsForSlider
-        self.brailleGenerators[rolenames.ROLE_SPIN_BUTTON]         = \
+        self.brailleGenerators[pyatspi.ROLE_SPIN_BUTTON]         = \
              self._getBrailleRegionsForSpinButton
-        self.brailleGenerators[rolenames.ROLE_SPLIT_PANE]          = \
+        self.brailleGenerators[pyatspi.ROLE_SPLIT_PANE]          = \
              self._getBrailleRegionsForSplitPane
-        self.brailleGenerators[rolenames.ROLE_TABLE]               = \
+        self.brailleGenerators[pyatspi.ROLE_TABLE]               = \
              self._getBrailleRegionsForTable
-        self.brailleGenerators[rolenames.ROLE_TABLE_CELL]          = \
+        self.brailleGenerators[pyatspi.ROLE_TABLE_CELL]          = \
              self._getBrailleRegionsForTableCellRow
-        self.brailleGenerators[rolenames.ROLE_TABLE_COLUMN_HEADER] = \
+        self.brailleGenerators[pyatspi.ROLE_TABLE_COLUMN_HEADER] = \
              self._getBrailleRegionsForTableColumnHeader
-        self.brailleGenerators[rolenames.ROLE_TABLE_ROW_HEADER]    = \
+        self.brailleGenerators[pyatspi.ROLE_TABLE_ROW_HEADER]    = \
              self._getBrailleRegionsForTableRowHeader
-        self.brailleGenerators[rolenames.ROLE_TEAR_OFF_MENU_ITEM]  = \
+        self.brailleGenerators[pyatspi.ROLE_TEAROFF_MENU_ITEM]  = \
              self._getBrailleRegionsForMenu
-        self.brailleGenerators[rolenames.ROLE_TERMINAL]            = \
+        self.brailleGenerators[pyatspi.ROLE_TERMINAL]            = \
              self._getBrailleRegionsForTerminal
-        self.brailleGenerators[rolenames.ROLE_TEXT]                = \
+        self.brailleGenerators[pyatspi.ROLE_TEXT]                = \
              self._getBrailleRegionsForText
-        self.brailleGenerators[rolenames.ROLE_TOGGLE_BUTTON]       = \
+        self.brailleGenerators[pyatspi.ROLE_TOGGLE_BUTTON]       = \
              self._getBrailleRegionsForToggleButton
-        self.brailleGenerators[rolenames.ROLE_TOOL_BAR]            = \
+        self.brailleGenerators[pyatspi.ROLE_TOOL_BAR]            = \
              self._getBrailleRegionsForToolBar
-        self.brailleGenerators[rolenames.ROLE_TREE]                = \
+        self.brailleGenerators[pyatspi.ROLE_TREE]                = \
              self._getBrailleRegionsForTable
-        self.brailleGenerators[rolenames.ROLE_TREE_TABLE]          = \
+        self.brailleGenerators[pyatspi.ROLE_TREE_TABLE]          = \
              self._getBrailleRegionsForTable
-        self.brailleGenerators[rolenames.ROLE_WINDOW]              = \
+        self.brailleGenerators[pyatspi.ROLE_WINDOW]              = \
              self._getBrailleRegionsForWindow
 
     def _getTextForAccelerator(self, obj):
@@ -199,7 +193,8 @@ class BrailleGenerator:
         Returns a string to be displayed.
         """
 
-        if obj.state.count(atspi.Accessibility.STATE_SENSITIVE) == 0:
+        state = obj.getState()
+        if not state.contains(pyatspi.STATE_SENSITIVE):
             # Translators: this represents an item on the screen that has
             # been set insensitive (or grayed out).
             #
@@ -207,63 +202,11 @@ class BrailleGenerator:
         else:
             return None
 
-    def _getTextForValue(self, obj):
-        """Returns the text to be displayed for the object's current value.
-
-        Arguments:
-        - obj: the Accessible object that may or may not have a value.
-
-        Returns a string representing the value.
-        """
-
-        value = obj.value
-
-        if not value:
-            return ""
-
-        # OK, this craziness is all about trying to figure out the most
-        # meaningful formatting string for the floating point values.
-        # The number of places to the right of the decimal point should
-        # be set by the minimumIncrement, but the minimumIncrement isn't
-        # always set.  So...we'll default the minimumIncrement to 1/100
-        # of the range.  But, if max == min, then we'll just go for showing
-        # them off to two meaningful digits.
-        #
-        try:
-            minimumIncrement = value.minimumIncrement
-        except:
-            minimumIncrement = 0.0
-
-        if minimumIncrement == 0.0:
-            minimumIncrement = (value.maximumValue - value.minimumValue) \
-                               / 100.0
-
-        try:
-            decimalPlaces = max(0, -math.log10(minimumIncrement))
-        except:
-            try:
-                decimalPlaces = max(0, -math.log10(value.minimumValue))
-            except:
-                try:
-                    decimalPlaces = max(0, -math.log10(value.maximumValue))
-                except:
-                    decimalPlaces = 0
-
-        formatter = "%%.%df" % decimalPlaces
-        valueString = formatter % value.currentValue
-        #minString   = formatter % value.minimumValue
-        #maxString   = formatter % value.maximumValue
-
-        # [[[TODO: WDW - probably want to do this as a percentage at some
-        # point?  Logged as bugzilla bug 319743.]]]
-        #
-        return valueString
-
-    def _getTextForRole(self, obj):
+    def _getTextForRole(self, obj, role=None):
         if (settings.brailleVerbosityLevel \
             == settings.VERBOSITY_LEVEL_VERBOSE)\
-           and (obj.role != rolenames.ROLE_UNKNOWN):
-            return rolenames.getBrailleForRoleName(obj)
+           and (obj.getRole() != pyatspi.ROLE_UNKNOWN):
+            return rolenames.getBrailleForRoleName(obj, role)
         else:
             return None
 
@@ -281,9 +224,9 @@ class BrailleGenerator:
         debug.println(debug.LEVEL_FINER,
                       "           obj             = %s" % obj.name)
         debug.println(debug.LEVEL_FINER,
-                      "           role            = %s" % obj.role)
+                      "           role            = %s" % obj.getRoleName())
 
-    def _getDefaultBrailleRegions(self, obj):
+    def _getDefaultBrailleRegions(self, obj, role=None):
         """Gets text to be displayed for the current object's name,
         role, and any accelerators.  This is usually the fallback
         braille generator should no other specialized braille
@@ -291,6 +234,7 @@ class BrailleGenerator:
 
         Arguments:
         - obj: an Accessible
+        - role: Role to use as an override.
 
         Returns a list where the first element is a list of Regions to
         display and the second element is the Region which should get
@@ -306,8 +250,9 @@ class BrailleGenerator:
             text, self._script.getDisplayedLabel(obj))
         text = self._script.appendString(
             text, self._script.getDisplayedText(obj))
-        text = self._script.appendString(text, self._getTextForValue(obj))
-        text = self._script.appendString(text, self._getTextForRole(obj))
+        text = self._script.appendString(text, 
+                                         self._script.getTextForValue(obj))
+        text = self._script.appendString(text, self._getTextForRole(obj, role))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -387,19 +332,27 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckBox", obj)
 
-        text = ""
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_INDETERMINATE):
+            indicatorindex = 2
+        elif state.contains(pyatspi.STATE_CHECKED):
+            indicatorindex = 1
+        else:
+            indicatorindex = 0
 
+        text = ""
+        state = obj.getState()
         text = self._script.appendString(
             text,
-            settings.brailleCheckBoxIndicators[
-                obj.state.count(atspi.Accessibility.STATE_CHECKED)])
+            settings.brailleCheckBoxIndicators[indicatorindex])
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
         text = self._script.appendString(
             text, self._script.getDisplayedText(obj))
 
-        text = self._script.appendString(text, self._getTextForRole(obj))
+        text = self._script.appendString(
+            text, self._getTextForRole(obj, pyatspi.ROLE_CHECK_BOX))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -420,12 +373,19 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForCheckMenuItem", obj)
 
-        text = ""
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_INDETERMINATE):
+            indicatorindex = 2
+        elif state.contains(pyatspi.STATE_CHECKED):
+            indicatorindex = 1
+        else:
+            indicatorindex = 0
 
+        text = ""
+        state = obj.getState()
         text = self._script.appendString(
             text,
-            settings.brailleCheckBoxIndicators[
-                obj.state.count(atspi.Accessibility.STATE_CHECKED)])
+            settings.brailleCheckBoxIndicators[indicatorindex])
 
         text = self._script.appendString(
             text, self._script.getDisplayedLabel(obj))
@@ -608,9 +568,12 @@ class BrailleGenerator:
             text, self._script.getDisplayedLabel(obj))
         text = self._script.appendString(
             text, self._script.getDisplayedText(obj))
-
-        if obj.image:
-            description = obj.image.imageDescription
+        try:
+            image = obj.queryImage()
+        except NotImplementedError:
+            pass
+        else:
+            description = image.imageDescription
             if len(description):
                 text = self._script.appendString(text, description)
 
@@ -634,7 +597,7 @@ class BrailleGenerator:
 
         self._debugGenerator("_getBrailleRegionsForImage", obj)
 
-        return self._getDefaultBrailleRegions(obj)
+        return self._getDefaultBrailleRegions(obj, pyatspi.ROLE_IMAGE)
 
     def _getBrailleRegionsForLabel(self, obj):
         """Get the braille for a label.
@@ -668,6 +631,59 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForListItem(self, obj):
+        """Get the braille for a listitem.
+
+        Arguments:
+        - obj: the listitem
+
+        """
+
+        self._debugGenerator("_getBrailleRegionsForListItem", obj)
+
+        text = ""
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_EXPANDABLE):
+            if state.contains(pyatspi.STATE_EXPANDED):
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
+                text = self._script.appendString(text, _('expanded'))
+            else:
+                # Translators: this represents the state of a node in a tree.
+                # 'expanded' means the children are showing.
+                # 'collapsed' means the children are not showing.
+                #
+                text = self._script.appendString(text, _('collapsed'))
+
+        if obj == orca_state.locusOfFocus:
+            text = self._script.appendString(
+                text, self._getTextForRole(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAccelerator(obj), "")
+
+        level = self._script.getNodeLevel(obj)
+        if level >= 0:
+            # Translators: this represents the depth of a node in a tree
+            # view (i.e., how many ancestors a node has).
+            #
+            text = self._script.appendString(text, _("LEVEL %d") \
+                                             % (level + 1))
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
+
     def _getBrailleRegionsForMenu(self, obj):
         """Get the braille for a menu.
 
@@ -736,7 +752,8 @@ class BrailleGenerator:
         # menu item" that is currently unchecked and braille that state.
         # See Orca bug #433398 for more details.
         #
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_CHECKED):
             text = self._script.appendString(
                 text,
                 settings.brailleCheckBoxIndicators[1])
@@ -940,10 +957,10 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
         text = ""
-
+        state = obj.getState()
         text = self._script.appendString(
             settings.brailleRadioButtonIndicators[
-                obj.state.count(atspi.Accessibility.STATE_CHECKED)],
+                int(state.contains(pyatspi.STATE_CHECKED))],
             text)
 
         text = self._script.appendString(
@@ -981,9 +998,10 @@ class BrailleGenerator:
 
         text = ""
 
+        state = obj.getState()
         text = self._script.appendString(
             settings.brailleRadioButtonIndicators[
-                obj.state.count(atspi.Accessibility.STATE_CHECKED)],
+                state.contains(pyatspi.STATE_CHECKED)],
             text)
 
         text = self._script.appendString(
@@ -1057,7 +1075,8 @@ class BrailleGenerator:
         # Ignore the text on the slider.
         #text = self._script.appendString(
         #    text, self._script.getDisplayedText(obj))
-        text = self._script.appendString(text, self._getTextForValue(obj))
+        text = self._script.appendString(text,
+                                         self._script.getTextForValue(obj))
         text = self._script.appendString(text, self._getTextForRole(obj))
 
         regions = []
@@ -1133,9 +1152,12 @@ class BrailleGenerator:
         if obj.childCount == 2:
             cellOrder = []
             hasToggle = [ False, False ]
-            for i in range(0, obj.childCount):
-                action = obj.child(i).action
-                if action:
+            for i, child in enumerate(obj):
+                try:
+                    action = child.queryAction()
+                except:
+                    continue
+                else:
                     for j in range(0, action.nActions):
                         if action.getName(j) == "toggle":
                             hasToggle[i] = True
@@ -1148,7 +1170,7 @@ class BrailleGenerator:
             if cellOrder:
                 for i in cellOrder:
                     [cellRegions, focusRegion] = \
-                            self._getBrailleRegionsForTableCell(obj.child(i))
+                            self._getBrailleRegionsForTableCell(obj[i])
                     if len(regions):
                         regions.append(braille.Region(" "))
                     else:
@@ -1162,14 +1184,16 @@ class BrailleGenerator:
         # state, check the NODE_CHILD_OF relation, etc.  Logged as
         # bugzilla bug 319750.]]]
         #
-        action = obj.action
-        if action:
+        try:
+            action = obj.queryAction()
+        except NotImplementedError:
+            pass
+        else:
             for i in range(0, action.nActions):
                 debug.println(debug.LEVEL_FINEST,
                     "braillegenerator._getBrailleRegionsForTableCell " \
                     + "looking at action %d" % i)
                 if action.getName(i) == "toggle":
-                    obj.role = rolenames.ROLE_CHECK_BOX
                     regions = self._getBrailleRegionsForCheckBox(obj)
 
                     # If this table cell doesn't have any label associated
@@ -1178,15 +1202,14 @@ class BrailleGenerator:
                     #
                     label = self._script.getDisplayedText( \
                         self._script.getRealActiveDescendant(obj))
-                    action = obj.action
+
                     if label == None or len(label) == 0:
-                        n = obj.parent.table.getColumnAtIndex(obj.index)
-                        header = obj.parent.table.getColumnHeader(n)
-                        accHeader = atspi.Accessible.makeAccessible(header)
+                        table = obj.parent.queryTable()
+                        n = table.getColumnAtIndex(obj.getIndexInParent())
+                        accHeader = table.getColumnHeader(n)
                         regions[0].append(braille.Region(" "))
                         regions[0].append(braille.Region(accHeader.name))
 
-                    obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
         if len(regions) == 0:
@@ -1206,19 +1229,22 @@ class BrailleGenerator:
         #
         displayedText = self._script.getDisplayedText( \
                           self._script.getRealActiveDescendant(obj))
-        if (not displayedText or len(displayedText) == 0) and obj.image:
-            if obj.image.imageDescription:
-                regions[0].append(obj.image.imageDescription)
-            obj.role = rolenames.ROLE_IMAGE
+        try:
+            image = obj.queryImage()
+        except:
+            image = None
+        if (not displayedText or len(displayedText) == 0) and image:
+            if image.imageDescription:
+                regions[0].append(image.imageDescription)
             [cellRegions, focusRegion] = self._getBrailleRegionsForImage(obj)
             regions[0].extend(cellRegions)
-            obj.role = rolenames.ROLE_TABLE_CELL
 
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
         #
-        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
-            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_EXPANDABLE):
+            if state.contains(pyatspi.STATE_EXPANDED):
                 # Translators: this represents the state of a node in a tree.
                 # 'expanded' means the children are showing.
                 # 'collapsed' means the children are not showing.
@@ -1252,16 +1278,19 @@ class BrailleGenerator:
         # valid table. It's possible that the parent could be a
         # table cell too (see bug #351501).
         #
-        if settings.readTableCellRow and obj.parent.table \
+        try:
+            table = obj.parent.queryTable()
+        except NotImplementedError:
+            table = None
+        if settings.readTableCellRow and table \
             and (not self._script.isLayoutOnly(obj.parent)):
             rowRegions = []
             savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
             settings.brailleVerbosityLevel = \
                                          settings.VERBOSITY_LEVEL_BRIEF
 
-            parent = obj.parent
-            row = parent.table.getRowAtIndex(obj.index)
-            column = parent.table.getColumnAtIndex(obj.index)
+            row = table.getRowAtIndex(obj.getIndexInParent())
+            column = table.getColumnAtIndex(obj.getIndexInParent())
 
             # This is an indication of whether we should speak all the
             # table cells (the user has moved focus up or down a row),
@@ -1269,19 +1298,19 @@ class BrailleGenerator:
             # the same row).
             #
             speakAll = True
-            if parent.__dict__.has_key("lastRow") and \
-                parent.__dict__.has_key("lastColumn"):
-                speakAll = (parent.lastRow != row) or \
-                       ((row == 0 or row == parent.table.nRows-1) and \
-                        parent.lastColumn == column)
+            if self._script.pointOfReference.has_key("lastRow") and \
+                self._script.pointOfReference.has_key("lastColumn"):
+                pointOfReference = self._script.pointOfReference
+                speakAll = (pointOfReference["lastRow"] != row) or \
+                       ((row == 0 or row == table.nRows-1) and \
+                        pointOfReference["lastColumn"] == column)
 
             if speakAll:
                 focusRowRegion = None
-                for i in range(0, parent.table.nColumns):
-                    accRow = parent.table.getAccessibleAt(row, i)
-                    cell = atspi.Accessible.makeAccessible(accRow)
-                    showing = cell.state.count( \
-                                    atspi.Accessibility.STATE_SHOWING)
+                for i in range(0, table.nColumns):
+                    cell = table.getAccessibleAt(row, i)
+                    state = cell.getState()
+                    showing = state.contains(pyatspi.STATE_SHOWING)
                     if showing:
                         [cellRegions, focusRegion] = \
                             self._getBrailleRegionsForTableCell(cell)
@@ -1483,20 +1512,21 @@ class BrailleGenerator:
         if settings.enableBrailleGrouping and groupChildren:
             parent = obj.parent
             isChild = parent \
-                      and ((parent.role == rolenames.ROLE_MENU) \
-                           or (parent.role == rolenames.ROLE_MENU_BAR) \
-                           or (parent.role == rolenames.ROLE_PAGE_TAB_LIST))
+                      and ((parent.getRole() == pyatspi.ROLE_MENU) \
+                           or (parent.getRole() == pyatspi.ROLE_MENU_BAR) \
+                           or (parent.getRole() == pyatspi.ROLE_PAGE_TAB_LIST))
             if isChild:
                 obj = parent
                 reallyGroupChildren = True
             else:
                 reallyGroupChildren = \
-                    (obj.role == rolenames.ROLE_MENU) \
-                    or (obj.role == rolenames.ROLE_MENU_BAR) \
-                    or (obj.role == rolenames.ROLE_PAGE_TAB_LIST)
+                    (obj.getRole() == pyatspi.ROLE_MENU) \
+                    or (obj.getRole() == pyatspi.ROLE_MENU_BAR) \
+                    or (obj.getRole() == pyatspi.ROLE_PAGE_TAB_LIST)
 
-        if self.brailleGenerators.has_key(obj.role):
-            generator = self.brailleGenerators[obj.role]
+        role = obj.getRole()
+        if self.brailleGenerators.has_key(role):
+            generator = self.brailleGenerators[role]
         else:
             generator = self._getDefaultBrailleRegions
 
@@ -1506,23 +1536,26 @@ class BrailleGenerator:
 
         if reallyGroupChildren:
             regions.append(braille.Region(" "))
-            selection = obj.selection
-            for i in range(0, obj.childCount):
+            try:
+                selection = obj.querySelection()
+            except NotImplementedError:
+                selection = None
+            i = 0
+            for child in obj:
                 debug.println(debug.LEVEL_FINEST,
                     "braillegenerator.getBrailleRegions " \
                     + "looking at child %d" % i)
-                child = obj.child(i)
 
                 # [[[TODO: richb - Need to investigate further.
                 # Sometimes, for some unknown reason, the child is None.
                 # We now test for this, rather than cause a traceback.
                 #
-                if child and (child.role != rolenames.ROLE_SEPARATOR):
+                if child and (child.getRole() != pyatspi.ROLE_SEPARATOR):
 
                 # the following line has been removed because insensitive
                 # menu items can get focus in StarOffice.
                 #
-                # and child.state.count(atspi.Accessibility.STATE_SENSITIVE):
+                # and child.getState().contains(pyatspi.STATE_SENSITIVE):
 
                     if (i > 0) and (i < (self, obj.childCount - 1)):
                         regions.append(braille.Region(" _ "))
@@ -1538,8 +1571,9 @@ class BrailleGenerator:
                     # workaround to the way OOo handles its menu items.
                     #
                     if (selection and selection.isChildSelected(i)) \
-                       or child.state.count(atspi.Accessibility.STATE_ARMED):
+                       or child.getState().contains(pyatspi.STATE_ARMED):
                         selectedRegion = result[1]
+                i += 1
 
         return [regions, selectedRegion]
 
@@ -1557,7 +1591,7 @@ class BrailleGenerator:
 
         regions = []
         parent = obj.parent
-        if parent and (parent.role in self.SKIP_CONTEXT_ROLES):
+        if parent and (parent.getRole() in self.SKIP_CONTEXT_ROLES):
             parent = parent.parent
         while parent and (parent.parent != parent):
             # [[[TODO: WDW - we might want to include more things here
@@ -1565,9 +1599,9 @@ class BrailleGenerator:
             # page tab lists might be a nice thing to include. Logged
             # as bugzilla bug 319751.]]]
             #
-            if (parent.role != rolenames.ROLE_FILLER) \
-                and (parent.role != rolenames.ROLE_SECTION) \
-                and (parent.role != rolenames.ROLE_SPLIT_PANE) \
+            if (parent.getRole() != pyatspi.ROLE_FILLER) \
+                and (parent.getRole() != pyatspi.ROLE_SECTION) \
+                and (parent.getRole() != pyatspi.ROLE_SPLIT_PANE) \
                 and (not self._script.isLayoutOnly(parent)):
 
                 # Announce the label and text of the object in the hierarchy.
@@ -1586,7 +1620,7 @@ class BrailleGenerator:
             # container for the grouped objects.  When we detect this,
             # we add the label to the overall context.]]]
             #
-            if parent.role == rolenames.ROLE_FILLER:
+            if parent.getRole() == pyatspi.ROLE_FILLER:
                 label = self._script.getDisplayedLabel(parent)
                 if label and len(label) and not label.isspace():
                     regions.append(braille.Region(" "))
@@ -1604,12 +1638,16 @@ class BrailleGenerator:
         # we first show the row header then the column header.
         #
         parent = obj.parent
-        if parent and parent.table:
-            row = parent.table.getRowAtIndex(obj.index)
+        try:
+            table = parent.queryTable()
+        except (NotImplementedError, AttributeError):
+            table = None
+        if parent and table:
+            row = table.getRowAtIndex(obj.getIndexInParent())
             if (row >= 0) \
-                and (not obj.role in [rolenames.ROLE_ROW_HEADER,
-                                      rolenames.ROLE_TABLE_ROW_HEADER]):
-                desc = parent.table.getRowDescription(row)
+                and (not obj.getRole() in [pyatspi.ROLE_ROW_HEADER,
+                                           pyatspi.ROLE_TABLE_ROW_HEADER]):
+                desc = table.getRowDescription(row)
             else:
                 desc = None
             if desc and len(desc):
@@ -1618,19 +1656,19 @@ class BrailleGenerator:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleLong + " "
+                            pyatspi.ROLE_ROW_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_ROW_HEADER].brailleShort + " "
+                            pyatspi.ROLE_ROW_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
 
-            col = parent.table.getColumnAtIndex(obj.index)
+            col = table.getColumnAtIndex(obj.getIndexInParent())
             if (col >= 0) \
-                and (not obj.role in [rolenames.ROLE_COLUMN_HEADER,
-                                      rolenames.ROLE_TABLE_COLUMN_HEADER]):
-                desc = parent.table.getColumnDescription(col)
+                and (not obj.getRole() in [pyatspi.ROLE_COLUMN_HEADER,
+                                           pyatspi.ROLE_TABLE_COLUMN_HEADER]):
+                desc = table.getColumnDescription(col)
             else:
                 desc = None
             if desc and len(desc):
@@ -1639,10 +1677,10 @@ class BrailleGenerator:
                     if brailleRolenameStyle \
                            == settings.BRAILLE_ROLENAME_STYLE_LONG:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleLong + " "
+                            pyatspi.ROLE_COLUMN_HEADER].brailleLong + " "
                     else:
                         text = desc + " " + rolenames.rolenames[\
-                            rolenames.ROLE_COLUMN_HEADER].brailleShort + " "
+                            pyatspi.ROLE_COLUMN_HEADER].brailleShort + " "
                 else:
                     text = desc
                 regions.append(braille.Region(text))
