@@ -20,9 +20,9 @@
 """Provides a graphical braille display at the top of the screen.
 This is mainly for debugging and for demonstrations."""
 
-__id__        = "$Id: brlmon.py 2882 2007-09-19 17:01:38Z richb $"
-__version__   = "$Revision: 2882 $"
-__date__      = "$Date: 2007-09-19 13:01:38 -0400 (Wed, 19 Sep 2007) $"
+__id__        = "$Id: brlmon.py 3322 2007-12-02 21:17:53Z wwalker $"
+__version__   = "$Revision: 3322 $"
+__date__      = "$Date: 2007-12-02 16:17:53 -0500 (Sun, 02 Dec 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -78,7 +78,8 @@ class BrlMon(gtk.Window):
         self.add(hbox)
         self.cellFrames = []
         self.cellLabels = []
-        for i in range(0, numCells):
+        i = 0
+        while (i < numCells):
             frame = gtk.Frame()
             frame.set_shadow_type(gtk.SHADOW_OUT)
             label = gtk.Label(" ")
@@ -87,17 +88,25 @@ class BrlMon(gtk.Window):
             hbox.add(frame)
             self.cellFrames.append(frame)
             self.cellLabels.append(label)
+            i += 1
 
         # This prevents it from getting focus.
         #
         self.set_property("accept-focus", False)
         self.connect_after("check-resize", self.onResize)
 
-    def onResize(self, object):
+    def onResize(self, obj):
         """Tell the window to be a dock and set its struts, which I
         thinks means to attempt to glue it somewhere on the display.
         """
 
+        # We know what we are doing here, so tell pylint not to flag
+        # the self.window method calls as errors (e.g., Class 'window' 
+        # has no 'get_size' member).  The disable-msg is localized to
+        # just this method.
+        #
+        # pylint: disable-msg=E1101
+
         screen_width = gtk.gdk.screen_width()
         [window_width, window_height] = self.window.get_size()
         if window_width < screen_width:
