@@ -29,10 +29,7 @@ __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006, 2007 Brailcom, o.p.s."
 __license__   = "LGPL"
 
-import threading
 import gobject
-import time
-import re
 
 import debug
 import speechserver
@@ -40,7 +37,6 @@ import settings
 import orca
 from acss import ACSS
 from orca_i18n import _
-from speechserver import VoiceFamily
 
 try:
     import speechd
@@ -49,7 +45,9 @@ except:
 else:    
     _speechd_available = True
     try:
-        from speechd import CallbackType
+        # Done this way to prevent a pychecker warning.
+        key = "CallbackType"
+        setattr(speechd, key, getattr(speechd, key))
     except ImportError:
         _speechd_version_ok = False
     else:
@@ -92,26 +90,26 @@ class SpeechServer(speechserver.SpeechServer):
         return servers
     getSpeechServers = staticmethod(getSpeechServers)
 
-    def _getSpeechServer(cls, id):
+    def _getSpeechServer(cls, serverId):
         """Return an active server for given id.
 
         Attempt to create the server if it doesn't exist yet.  Returns None
         when it is not possible to create the server.
         
         """
-        if not cls._active_servers.has_key(id):
-            cls(id)
+        if not cls._active_servers.has_key(serverId):
+            cls(serverId)
         # Don't return the instance, unless it is succesfully added
         # to `_active_Servers'.
-        return cls._active_servers.get(id)
+        return cls._active_servers.get(serverId)
     _getSpeechServer = classmethod(_getSpeechServer)
 
     def getSpeechServer(info=None):
         if info is not None:
-            id = info[1]
+            thisId = info[1]
         else:
-            id = SpeechServer.DEFAULT_SERVER_ID
-        return SpeechServer._getSpeechServer(id)
+            thisId = SpeechServer.DEFAULT_SERVER_ID
+        return SpeechServer._getSpeechServer(thisId)
     getSpeechServer = staticmethod(getSpeechServer)
 
     def shutdownActiveServers():
@@ -121,8 +119,10 @@ class SpeechServer(speechserver.SpeechServer):
 
     # *** Instance methods ***
 
-    def __init__(self, id):
-        self._id = id
+    def __init__(self, serverId):
+        speechserver.SpeechServer.__init__(self)
+
+        self._id = serverId
         self._acss_manipulators = (
             (ACSS.RATE, self._set_rate),
             (ACSS.AVERAGE_PITCH, self._set_pitch),
@@ -151,6 +151,12 @@ class SpeechServer(speechserver.SpeechServer):
             speechd.CallbackType.END: speechserver.SayAllContext.COMPLETED,
            #speechd.CallbackType.INDEX_MARK:speechserver.SayAllContext.PROGRESS,
             }
+
+        # Initialize variables to None to make pylint happy.
+        #
+        self._current_voice_properties = None
+        self._client = None
+
         # Translators: This string will appear in the list of
         # available voices for the current speech engine.  %s will be
         # replaced by the name of the current speech engine, such as
@@ -160,7 +166,7 @@ class SpeechServer(speechserver.SpeechServer):
         # the list will contain the names of all available "real"
         # voices provided by the speech engine.
         #
-        self._default_voice_name = _("%s default voice") % id
+        self._default_voice_name = _("%s default voice") % serverId
         
         try:
             self._init()
@@ -168,7 +174,7 @@ class SpeechServer(speechserver.SpeechServer):
             debug.println(debug.LEVEL_WARNING,
                           "Speech Dispatcher service failed to connect: %s" % e)
         else:
-            self.__class__._active_servers[id] = self
+            self.__class__._active_servers[serverId] = self
 
     def _init(self):
         self._client = client = speechd.SSIPClient('Orca', component=self._id)
@@ -205,25 +211,32 @@ class SpeechServer(speechserver.SpeechServer):
         self._send_command(self._client.set_volume, volume)
 
     def _set_family(self, acss_family):
-        locale = acss_family[VoiceFamily.LOCALE]
+        locale = acss_family[speechserver.VoiceFamily.LOCALE]
         if locale:
             lang = locale.split('_')[0]
             if lang and len(lang) == 2:
                 self._send_command(self._client.set_language, lang)
-        name = acss_family[VoiceFamily.NAME]
-        if name != self._default_voice_name:
-            self._send_command(self._client.set_synthesis_voice, name)
+        try:
+            # This command is not available with older SD versions.
+            set_synthesis_voice = self._client.set_synthesis_voice
+        except AttributeError:
+            pass
+        else:
+            name = acss_family[speechserver.VoiceFamily.NAME]
+            if name != self._default_voice_name:
+                self._send_command(set_synthesis_voice, name)
             
     def _apply_acss(self, acss):
         current = self._current_voice_properties
-        for property, method in self._acss_manipulators:
-            value = acss.get(property)
-            if value is not None and current.get(property) != value:
+        for thisProperty, method in self._acss_manipulators:
+            value = acss.get(thisProperty)
+            if value is not None and current.get(thisProperty) != value:
                 method(value)
-                current[property] = value
+                current[thisProperty] = value
 
     def _speak(self, text, acss, **kwargs):
-        self._apply_acss(acss)
+        if acss is not None:
+            self._apply_acss(acss)
         self._send_command(self._client.speak, text, **kwargs)
 
     def _say_all(self, iterator, orca_callback):
@@ -237,11 +250,11 @@ class SpeechServer(speechserver.SpeechServer):
         except StopIteration:
             pass
         else:
-            def callback(type, index_mark=None):
+            def callback(callbackType, index_mark=None):
                 # This callback is called in Speech Dispatcher listener thread.
                 # No subsequent Speech Dispatcher interaction is allowed here,
                 # so we pass the calls to the gidle thread.
-                t = self._CALLBACK_TYPE_MAP[type]
+                t = self._CALLBACK_TYPE_MAP[callbackType]
                 if t == speechserver.SayAllContext.PROGRESS:
                     if index_mark:
                         context.currentOffset = int(index_mark)
@@ -291,11 +304,18 @@ class SpeechServer(speechserver.SpeechServer):
             lang = None
         else:
             lang = locale.split('_')[0]
-        voices = ((self._default_voice_name, lang, None),) + \
-                 self._send_command(self._client.list_synthesis_voices)
-        families = [VoiceFamily({VoiceFamily.NAME: name,
-                                 #VoiceFamily.GENDER: VoiceFamily.MALE,
-                                 VoiceFamily.LOCALE: lang})
+        voices = ((self._default_voice_name, lang, None),)
+        try:
+            # This command is not available with older SD versions.
+            list_synthesis_voices = self._client.list_synthesis_voices
+        except AttributeError:
+            pass
+        else:
+            voices += self._send_command(list_synthesis_voices)
+        families = [speechserver.VoiceFamily({ \
+              speechserver.VoiceFamily.NAME: name,
+              #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
+              speechserver.VoiceFamily.LOCALE: lang})
                     for name, lang, dialect in voices]
         return families
 
@@ -309,10 +329,10 @@ class SpeechServer(speechserver.SpeechServer):
         if text:
             self._speak(text, acss)
 
-    def speakUtterances(self, list, acss=None, interrupt=True):
+    def speakUtterances(self, utteranceList, acss=None, interrupt=True):
         #if interrupt:
         #    self._cancel()
-        for utterance in list:
+        for utterance in utteranceList:
             if utterance:
                 self._speak(utterance, acss)
 
@@ -322,8 +342,8 @@ class SpeechServer(speechserver.SpeechServer):
     def speakCharacter(self, character, acss=None):
         self._send_command(self._client.char, character)
 
-    def speakKeyEvent(self, event_string, type):
-        if type == orca.KeyEventType.PRINTABLE:
+    def speakKeyEvent(self, event_string, eventType):
+        if eventType == orca.KeyEventType.PRINTABLE:
             # We currently only handle printable characters by Speech
             # Dispatcher's KEY command.  For other keys, such as Ctrl, Shift
             # etc. we prefer Orca's verbalization.
@@ -332,10 +352,12 @@ class SpeechServer(speechserver.SpeechServer):
             else:
                 voice = settings.voices[settings.DEFAULT_VOICE]
             key = self.KEY_NAMES.get(event_string, event_string)
-            self._apply_acss(voice)
+            if voice is not None:
+                self._apply_acss(voice)
             self._send_command(self._client.key, key)
         else:
-            return super(SpeechServer, self).speakKeyEvent(event_string, type)
+            return super(SpeechServer, self).speakKeyEvent(event_string, 
+                                                           eventType)
 
     def increaseSpeechRate(self, step=5):
         self._change_default_speech_rate()
